using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.IO;
using System.Net;

namespace Google_Maps_Download
{
    public partial class SelectDetail4x : Form
    {
        public SelectDetail4x(int zoom, int min_zoom, int x, int y, string map_type, mapURL MU, WebClient WC)
        {
            InitializeComponent();

            mu = MU;

            wc = WC;

            Zoom = zoom;
            MinZoom = min_zoom;
            X = x;
            Y = y;
            MapType = map_type;
        }

        private int Zoom;
        private int MinZoom;
        private int X;
        private int Y;
        private string MapType;

        public mapURL mu;

        private WebClient wc;

        private void SelectDetail4x_Load(object sender, EventArgs e)
        {
            int max_x_zoom = (int)Math.Pow(2, Zoom - MinZoom);
            int offset = 1;
            while (max_x_zoom >= 1)
            {
                CheckBox cb = new CheckBox();
                cb.Location = new Point(0, 20 * offset);
                cb.Checked = true;
                cb.Width = 50;
                cb.Text = max_x_zoom.ToString() + "x";
                cb.Name = "zl" + ((Zoom - MinZoom) - offset + 1).ToString();
                if (offset == 1 || max_x_zoom == 1)
                    cb.Enabled = false;
                this.Controls.Add(cb);

                offset++;
                max_x_zoom = max_x_zoom / 2;
            }

            for (int i = 0; i < 8; i++)
            {
                for (int j = 0; j < 8; j++)
                {
                    CheckBox cb = new CheckBox();
                    cb.Location = new Point(0 + i * 128 + 64, 24 + j * 128 + 64);
                    cb.Checked = true;
                    cb.Width = 20;
                    cb.Name = "cb" + (X * 4 + i).ToString() + "_" + (Y * 4 + j).ToString();
                    this.Controls.Add(cb);

                    PictureBox pb = new PictureBox();
                    pb.Location = new Point(0 + i * 128, 24 + j * 128);
                    pb.Width = 128;
                    pb.Height = 128;
                    pb.SizeMode = PictureBoxSizeMode.StretchImage;
                    pb.BorderStyle = BorderStyle.FixedSingle;

                    pb.Image = new Bitmap(new MemoryStream(wc.DownloadData(mu.MapByType(Zoom - 2, X * 4 + i, Y * 4 + j, MapType))));

                    pb.Name = "pb" + (X * 4 + i).ToString() + "_" + (Y * 4 + j).ToString();
                    pb.Click += new EventHandler(pb_Click);
                    
                    this.Controls.Add(pb);
                }
            }

            
        }

        void pb_Click(object sender, EventArgs e)
        {
            foreach (Control cn in this.Controls)
            {
                if (cn.Name == ((PictureBox)sender).Name.Replace("pb", "cb"))
                {
                    if (((CheckBox)cn).Checked)
                        ((CheckBox)cn).Checked = false;
                    else
                        ((CheckBox)cn).Checked = true;
                }
            }
        }

        public DataTable TilesToIgnore;

        private void Done_Click(object sender, EventArgs e)
        {
            TilesToIgnore = new DataTable();
            TilesToIgnore.Columns.Add("zoom", typeof(Int32));
            TilesToIgnore.Columns.Add("x", typeof(Int32));
            TilesToIgnore.Columns.Add("y", typeof(Int32));

            foreach (Control cn in this.Controls)
            {
                if (cn.Name.StartsWith("cb"))
                {
                    if (((CheckBox)cn).Checked == false)
                    {
                        int x = Int32.Parse(cn.Name.Substring(2, cn.Name.IndexOf("_") - 2));
                        int y = Int32.Parse(cn.Name.Substring(cn.Name.IndexOf("_") + 1, cn.Name.Length - 1 - cn.Name.IndexOf("_")));
                        int zoom_span = 2;
                        for (int zoom_off = 3; Zoom - zoom_off >= MinZoom; zoom_off++, zoom_span*=2)
                        {
                            for (int x_span = 0; x_span < zoom_span; x_span++)
                            {
                                for (int y_span = 0; y_span < zoom_span; y_span++)
                                {
                                    DataRow new_row = TilesToIgnore.NewRow();
                                    new_row["zoom"] = Zoom - zoom_off;
                                    new_row["x"] = x * zoom_span + x_span;
                                    new_row["y"] = y * zoom_span + y_span;
                                    TilesToIgnore.Rows.Add(new_row);
                                }
                            }
                        }
                    }
                }
            }

            // THE LAYERS (if not already exluded)
            foreach (Control cn in this.Controls)
            {
                if (cn.Name.StartsWith("zl"))
                {
                    if (((CheckBox)cn).Checked == false)
                    {
                        int zoom_off_layer = Int32.Parse(cn.Name.Substring(2, cn.Name.Length - 2));
                        int zoom_span = 2;
                        for (int zoom_off = 0; Zoom - zoom_off >= MinZoom; zoom_off++, zoom_span *= 2)
                        {
                            if (zoom_off == zoom_off_layer)
                            {
                                for (int x_span = 0; x_span < zoom_span; x_span++)
                                {
                                    for (int y_span = 0; y_span < zoom_span; y_span++)
                                    {
                                        if (TilesToIgnore.Select("zoom = " + (Zoom - zoom_off).ToString()
                                            + " AND x = " + (X * zoom_span / 2 + x_span).ToString()
                                            + " AND y = " + (Y * zoom_span / 2 + y_span).ToString()).Length == 0)
                                        {
                                            DataRow new_row = TilesToIgnore.NewRow();
                                            new_row["zoom"] = Zoom - zoom_off;
                                            new_row["x"] = X * zoom_span / 2 + x_span;
                                            new_row["y"] = Y * zoom_span / 2 + y_span;
                                            TilesToIgnore.Rows.Add(new_row);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }

            this.Close();
        }
    }
}